﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace DSwav2c
{
    public class WaveManager
    {
        public WavHeader Header;

        public struct WavHeader
        {
            public byte[] riffID; // "riff"
            public uint size;  // ファイルサイズ-8
            public byte[] wavID;  // "WAVE"
            public byte[] fmtID;  // "fmt "
            public uint fmtSize; // fmtチャンクのバイト数
            public ushort format; // フォーマット
            public ushort channels; // チャンネル数
            public uint sampleRate; // サンプリングレート
            public uint bytePerSec; // データ速度
            public ushort blockSize; // ブロックサイズ
            public ushort bit;  // 量子化ビット数
            public byte[] dataID; // "data"
            public uint dataSize; // 波形データのバイト数
        }

        public WaveManager()
        {
            Header = new WavHeader();
        }

        public String LoadAndGenerate(String inFileName, String inVariableName)
        {

            String aTextList;
            long aTemp;
            UInt16 aExtendSize;
            UInt32 aChunkSize;
            UInt32 aSampleLength;


            using (FileStream fs = new FileStream(inFileName, FileMode.Open, FileAccess.Read))
            using (BinaryReader br = new BinaryReader(fs))
            {
                try
                {
                    Header.riffID = br.ReadBytes(4);
                    Header.size = br.ReadUInt32();
                    Header.wavID = br.ReadBytes(4);
                    Header.fmtID = br.ReadBytes(4);
                    Header.fmtSize = br.ReadUInt32();
                    Header.format = br.ReadUInt16();
                    Header.channels = br.ReadUInt16();
                    Header.sampleRate = br.ReadUInt32();
                    Header.bytePerSec = br.ReadUInt32();
                    Header.blockSize = br.ReadUInt16();
                    Header.bit = br.ReadUInt16();

                    if (Header.fmtSize > 16)
                    {
                        aExtendSize = br.ReadUInt16();

                        /* 拡張部分 */
                        for (int j = 0; j < aExtendSize >> 1; j++)
                        {
                            aTemp = br.ReadUInt16();
                        }
                    }


                    //factチェック
                    Header.dataID = br.ReadBytes(4);

                    if( ( Header.dataID[0] == 0x66) && ( Header.dataID[1] == 0x61) && ( Header.dataID[2] == 0x63) && ( Header.dataID[3] == 0x74))
                    {
                        aChunkSize  = br.ReadUInt32();
                        aSampleLength = br.ReadUInt32();
                        Header.dataID = br.ReadBytes(4);

                    }


                    Header.dataSize = br.ReadUInt32();

                    if ((Header.format != 1) && (Header.format != 17))
                    {
                        MessageBox.Show("Format ID of the Wave file is not LPCM(1) orADPCM(17).", "Error",
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Error);
                    }


                    if (Header.sampleRate != 8000)
                    {
                        //8kHz以外
                        MessageBox.Show("Sample rate of the Wave file is not 8kHz.","Error",
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Error);

                    }


                    var listString = new StringBuilder();

                    listString.Append("// Generated by Desktop Station Wave to C converter");
                    listString.Append(Environment.NewLine);

                    switch (Header.format)
                    {
                        case 0x01:
                            listString.Append("// PCM type: Linear");
                            break;

                        case 0x11:
                            listString.Append("// PCM type: IMA ADPCM");
                            break;
                    }

                    listString.Append(Environment.NewLine);


                    listString.Append(Environment.NewLine);
                    listString.Append("const int " + inVariableName + "_sampleRate = 8000;");
                    listString.Append(Environment.NewLine);
                    listString.Append("const int " + inVariableName + "_length = " + Header.dataSize.ToString() + ";");
                    listString.Append(Environment.NewLine);
                    listString.Append("const signed char " + inVariableName +  "_data[] PROGMEM ={");


                    uint aCount = Header.dataSize / Header.blockSize;

                    if (Header.blockSize == 256)
                    {
                        aCount = Header.dataSize;
                    }



                    for (int i = 0; i < aCount; i++)
                    {

                        int aData;

                        switch ((Header.channels * 10) + (Header.bit >> 3))
                       {
                           case 10:
                               //Mono 4bit x2
                               aData = br.ReadByte();
                               break;
                            case 11:
                                //Mono 8bit
                                aData = br.ReadByte();
                                break;
                            case 12:
                                //Mono 16bit
                                aData = br.ReadInt16() >> 8;
                                break;
                            case 21:
                                //Stereo 8bit
                                aData = br.ReadByte();
                                br.ReadByte();//読み捨て
                                break;
                            case 22:
                                //Stereo 16bit
                                aData = br.ReadInt16() >> 8;
                                br.ReadInt16();//読み捨て
                                break;
                            default:
                                aData = 0;
                                break;
                        }
                        
                        
                        
                        listString.Append(aData.ToString() + ", ");

                        if (i % 20 == 0)
                        {
                            listString.Append(Environment.NewLine);
                        }



                    }

                    listString.Append(" };");
                   

                    aTextList  = listString.ToString();


                }
                finally
                {
                    if (br != null)
                    {
                        br.Close();
                    }
                    if (fs != null)
                    {
                        fs.Close();
                    }
                }
            }
            //Console.WriteLine(args[1] + "をオープンしました。");

            return aTextList;
        }


    }
}
